/*
 * Copyright (C) 2017 Aaron M. D. Jones <aaronmdjones@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INC_LIBACOPM_DNSBL_INTERNAL_H
#define INC_LIBACOPM_DNSBL_INTERNAL_H

#include "acopm-common.h"

#include <netinet/in.h> /* INET6_ADDRSTRLEN */

#include <event2/dns.h> /* struct evdns_base */
#include <event2/event.h> /* struct event_base */

#include "dnsbl/client.h" /* acopm_dnsbl_callback */
#include "dnsbl/domain.h" /* acopm_dnsbl_aftype */
#include "utils/siphash24.h" /* UT_hash_handle */

#define ACOPM_DNSBL_MAXHOST_LEN         253
#define ACOPM_DNSBL_MAXHOST_REV         64
#define ACOPM_DNSBL_NOMATCH_MASK        0x7F000000UL /* 127.0.0.0 */

struct acopm_dnsbl_client;

typedef void (*acopm_dnsbl_callback)(struct acopm_dnsbl_client *, const char *, const char *, const char *, const char *);

struct acopm_dnsbl_client
{
	struct evdns_base               *ev_base;
	struct event                    *gc_timer;
	struct acopm_dnsbl_domain       *domains;
	struct acopm_dnsbl_lookup       *lookups;
	bool                             ready:1;
} acopm_attr_packed;

struct acopm_dnsbl_domain
{
	struct acopm_dnsbl_domain       *prev;
	struct acopm_dnsbl_domain       *next;
	struct acopm_dnsbl_client       *client;
	struct acopm_dnsbl_match        *matches;
	enum acopm_dnsbl_aftype          type;
	char                             host[ACOPM_DNSBL_MAXHOST_LEN + 1];
} acopm_attr_packed;

struct acopm_dnsbl_lookup
{
	struct acopm_dnsbl_lookup       *prev;
	struct acopm_dnsbl_lookup       *next;
	struct acopm_dnsbl_client       *client;
	struct acopm_dnsbl_request      *requests;
	acopm_dnsbl_callback             callback;
	int                              family;
	char                             address[INET6_ADDRSTRLEN];
	char                             address_rev[ACOPM_DNSBL_MAXHOST_REV + 1];
} acopm_attr_packed;

struct acopm_dnsbl_match
{
	UT_hash_handle                   hh;
	struct acopm_dnsbl_domain       *domain;
	uint32_t                         addr;
	char                             desc[512];
} acopm_attr_packed;

struct acopm_dnsbl_request
{
	struct acopm_dnsbl_request      *prev;
	struct acopm_dnsbl_request      *next;
	struct acopm_dnsbl_domain       *domain;
	struct acopm_dnsbl_lookup       *lookup;
	struct evdns_request            *request;
} acopm_attr_packed;

#endif /* !INC_LIBACOPM_DNSBL_INTERNAL_H */
