/*
 * Copyright (C) 2017 Aaron M. D. Jones <aaronmdjones@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INC_LIBACOPM_IRC_H
#define INC_LIBACOPM_IRC_H

#include "acopm-common.h"

#include <event2/event.h> /* struct event_base */

#include "utils/bufconn.h" /* enum acopm_conn_certfp_method */

#define ACOPM_IRC_LINE_MAX              510U
#define ACOPM_IRC_NOTICE_MAX            480U

struct acopm_irc_client; /* definition in internal.h */

typedef void (*acopm_irc_connect_cb)(struct acopm_irc_client *, const char *, const char *);
typedef void (*acopm_irc_command_cb)(struct acopm_irc_client *, size_t, char **);
typedef void (*acopm_irc_restart_cb)(void);

struct acopm_irc_client * acopm_attr_malloc
acopm_irc_new(struct event_base *, acopm_irc_connect_cb, acopm_irc_command_cb, acopm_irc_restart_cb);

bool acopm_irc_add_host(struct acopm_irc_client *, const char *);
void acopm_irc_set_port(struct acopm_irc_client *, uint16_t);
bool acopm_irc_set_bind4(struct acopm_irc_client *, const struct sockaddr_in *);
bool acopm_irc_set_bind6(struct acopm_irc_client *, const struct sockaddr_in6 *);

#ifdef BUILD_WITH_MBEDTLS
void acopm_irc_set_tls(struct acopm_irc_client *, bool, enum acopm_conn_certfp_method);
bool acopm_irc_add_tls_certfp(struct acopm_irc_client *, const char *) acopm_attr_warn_unused;
void acopm_irc_set_tls_cert(struct acopm_irc_client *, const char *, const char *);
#endif

void acopm_irc_set_nick(struct acopm_irc_client *, const char *);
void acopm_irc_set_user(struct acopm_irc_client *, const char *);
void acopm_irc_set_pass(struct acopm_irc_client *, const char *);
void acopm_irc_set_real(struct acopm_irc_client *, const char *);
void acopm_irc_set_acct(struct acopm_irc_client *, const char *, const char *);
void acopm_irc_set_oper(struct acopm_irc_client *, const char *, const char *);
void acopm_irc_set_pcmd(struct acopm_irc_client *, const char *);
void acopm_irc_set_away(struct acopm_irc_client *, const char *);
void acopm_irc_set_mode(struct acopm_irc_client *, const char *);
void acopm_irc_set_chan(struct acopm_irc_client *, const char *, const char *, unsigned int);
bool acopm_irc_set_conn(struct acopm_irc_client *, const char *) acopm_attr_warn_unused;
void acopm_irc_set_conn_is_regexp(struct acopm_irc_client *, bool);
void acopm_irc_free(struct acopm_irc_client *);

// The following 4 functions located within internal.c
bool acopm_irc_connect(struct acopm_irc_client *);
bool acopm_irc_writev(struct acopm_irc_client *, const char *, ...) acopm_attr_printf(2, 3);
bool acopm_irc_log(struct acopm_irc_client *, const char *, ...) acopm_attr_printf(2, 3);
void acopm_irc_disconnect(struct acopm_irc_client *);

#endif /* !INC_LIBACOPM_IRC_H */
