/*
 * Copyright (C) 2017 Aaron M. D. Jones <aaronmdjones@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INC_LIBACOPM_PROXY_INTERNAL_H
#define INC_LIBACOPM_PROXY_INTERNAL_H

#include "acopm-common.h"

#include <time.h> /* time_t */

#include <netinet/in.h> /* INET6_ADDRSTRLEN */

#include <event2/event.h> /* struct event_base */

#include "proxy/client.h" /* struct acopm_proxy_* */
#include "proxy/host.h" /* acopm_proxy_callback */
#include "proxy/port.h" /* enum acopm_proxy_type */
#include "utils/bufconn.h" /* struct acopm_connection */

#define ACOPM_PROXY_CONNECT_TIMEOUT_DEF 10U
#define ACOPM_PROXY_DATA_TIMEOUT_DEF    10U
#define ACOPM_PROXY_RECVBUF_SZ_DEF      2048U
#define ACOPM_PROXY_SENDBUF_SZ_DEF      256U

struct acopm_proxy_client;
struct acopm_proxy_port;
struct acopm_proxy_host;
struct acopm_proxy_scan;

typedef bool (*acopm_proxy_setup_func)(struct acopm_proxy_scan *);

struct acopm_proxy_client
{
	struct event_base               *ev_base;
	struct acopm_proxy_port         *ports;
	struct acopm_proxy_host         *hosts;
	size_t                           recvbuf_sz;
	size_t                           sendbuf_sz;
	size_t                           username_length;
	time_t                           connect_timeout;
	time_t                           data_timeout;
	char                             username[ACOPM_PROXY_USER_MAX + 1];
} acopm_attr_packed;

struct acopm_proxy_port
{
	struct acopm_proxy_port         *prev;
	struct acopm_proxy_port         *next;
	struct acopm_proxy_client       *client;
	uint16_t                         port;
	enum acopm_proxy_type            type;
} acopm_attr_packed;

struct acopm_proxy_host
{
	struct acopm_proxy_host         *prev;
	struct acopm_proxy_host         *next;
	struct acopm_proxy_client       *client;
	struct acopm_proxy_scan         *scans;
	acopm_proxy_callback             callback;
	int                              address_family;
	unsigned char                    address_raw[INET6_ADDRLEN];
	char                             address_str[INET6_ADDRSTRLEN];
} acopm_attr_packed;

struct acopm_proxy_scan
{
	struct acopm_proxy_scan         *prev;
	struct acopm_proxy_scan         *next;
	struct acopm_proxy_host         *host;
	struct acopm_proxy_port         *port;
	struct acopm_connection         *conn;
	void                            *data;
} acopm_attr_packed;

#endif /* !INC_LIBACOPM_PROXY_INTERNAL_H */
