/*
 * Copyright (C) 2017 Aaron M. D. Jones <aaronmdjones@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INC_LIBACOPM_PROXY_SOCKS5_H
#define INC_LIBACOPM_PROXY_SOCKS5_H

#include "acopm-common.h"

#define ACOPM_SOCKS5_VERSION            0x05

#define ACOPM_SOCKS5_AUTHMETHOD_NONE    0x00

#define ACOPM_SOCKS5_ADDRTYPE_IPV4      0x01
#define ACOPM_SOCKS5_ADDRTYPE_IPV6      0x04
#define ACOPM_SOCKS5_ADDRTYPE_HOSTNAME  0x03

#define ACOPM_SOCKS5_CMD_CONNECT        0x01
#define ACOPM_SOCKS5_CMD_LISTEN         0x02
#define ACOPM_SOCKS5_CMD_UDP_ASSOC      0x03

#define ACOPM_SOCKS5_REQ_GRANTED        0x00
#define ACOPM_SOCKS5_REQ_FAILURE        0x01
#define ACOPM_SOCKS5_REQ_REJECTED       0x02
#define ACOPM_SOCKS5_REQ_NET_UNREACH    0x03
#define ACOPM_SOCKS5_REQ_HOST_UNREACH   0x04
#define ACOPM_SOCKS5_REQ_HOST_REFUSED   0x05
#define ACOPM_SOCKS5_REQ_TTL_EXPIRED    0x06
#define ACOPM_SOCKS5_REQ_CMD_ERROR      0x07
#define ACOPM_SOCKS5_REQ_ADDR_NOSUPPORT 0x08

struct acopm_socks5_methods
{
	uint8_t                         version;
	uint8_t                         methods;
} acopm_attr_packed;

struct acopm_socks5_method
{
	uint8_t                         version;
	uint8_t                         method;
} acopm_attr_packed;

struct acopm_socks5_request_vx
{
	uint8_t                         version;
	uint8_t                         command;
	uint8_t                         reserved;
	uint8_t                         addrtype;
} acopm_attr_packed;

struct acopm_socks5_response_vx
{
	uint8_t                         version;
	uint8_t                         status;
	uint8_t                         reserved;
	uint8_t                         addrtype;
} acopm_attr_packed;

struct acopm_socks5_request_v4
{
	struct acopm_socks5_request_vx  rx;
	uint32_t                        address;
	uint16_t                        port;
} acopm_attr_packed;

struct acopm_socks5_response_v4
{
	struct acopm_socks5_response_vx rx;
	uint32_t                        address;
	uint16_t                        port;
} acopm_attr_packed;

struct acopm_socks5_request_v6
{
	struct acopm_socks5_request_vx  rx;
	uint8_t                         address[INET6_ADDRLEN];
	uint16_t                        port;
} acopm_attr_packed;

struct acopm_socks5_response_v6
{
	struct acopm_socks5_response_vx rx;
	uint8_t                         address[INET6_ADDRLEN];
	uint16_t                        port;
} acopm_attr_packed;

union acopm_socks5_request
{
	struct acopm_socks5_request_vx  rx;
	struct acopm_socks5_request_v4  r4;
	struct acopm_socks5_request_v6  r6;
} acopm_attr_packed;

union acopm_socks5_response
{
	struct acopm_socks5_response_vx rx;
	struct acopm_socks5_response_v4 r4;
	struct acopm_socks5_response_v6 r6;
} acopm_attr_packed;

#endif /* !INC_LIBACOPM_PROXY_SOCKS5_H */
