/*
 * Copyright (C) 2017 Aaron M. D. Jones <aaronmdjones@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "acopm-common.h"

#include <assert.h> /* assert() */

#include "utils/base64.h" /* decls for own functions */

void
acopm_base64encode(const unsigned char *restrict src, size_t src_len, char *restrict dst)
{
	assert(src != NULL);
	assert(src_len > 0);
	assert(dst != NULL);

	static const char acopm_irc_base64_etable[] = {
		0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F, 0x50,
		0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66,
		0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76,
		0x77, 0x78, 0x79, 0x7A, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x2B, 0x2F
	};

	while (src_len > 2)
	{
		*dst++ = acopm_irc_base64_etable[(size_t) (src[0] >> 0x02)];
		*dst++ = acopm_irc_base64_etable[(size_t) (((src[0] & 0x03) << 0x04) + (src[1] >> 0x04))];
		*dst++ = acopm_irc_base64_etable[(size_t) (((src[1] & 0x0F) << 0x02) + (src[2] >> 0x06))];
		*dst++ = acopm_irc_base64_etable[(size_t) (src[2] & 0x3F)];

		src += 3;
		src_len -= 3;
	}

	if (src_len > 0)
	{
		*dst++ = acopm_irc_base64_etable[(size_t) (src[0] >> 0x02)];

		if (src_len > 1)
		{
			*dst++ = acopm_irc_base64_etable[(size_t) (((src[0] & 0x03) << 0x04) + (src[1] >> 0x04))];
			*dst++ = acopm_irc_base64_etable[(size_t) ((src[1] & 0x0F) << 0x02)];
			*dst++ = 0x3D;
		}
		else
		{
			*dst++ = acopm_irc_base64_etable[(size_t) ((src[0] & 0x03) << 0x04)];
			*dst++ = 0x3D;
			*dst++ = 0x3D;
		}
	}

	*dst++ = 0x00;
}
