/*
 * Copyright (C) 2017 Aaron M. D. Jones <aaronmdjones@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "acopm-common.h"

#include <errno.h> /* EINTR, ERESTART, EAGAIN, EWOULDBLOCK */

#include "utils/ignore_errno.h" /* decls for own functions */

bool
acopm_ignore_errno(const int error)
{
	switch (error)
	{

#ifdef EINTR
		case EINTR:
#endif
#ifdef ERESTART
		case ERESTART:
#endif

	/*
	 * This mess is necessary. It cannot be simplified while remaining portable.
	 * The issue is that implementations can define EWOULDBLOCK or they can define EAGAIN.
	 *
	 * However, the relevant standards do not state that you can't define both, and they also
	 * don't state that they can't have different values! Naturally, if they had the same
	 * value, having both in a switch() statement would be a syntax error.
	 */

#if defined(EAGAIN) && defined(EWOULDBLOCK)
#  if (EAGAIN != EWOULDBLOCK)
		case EAGAIN:
#  endif
		case EWOULDBLOCK:
#else
#  ifdef EAGAIN
		case EAGAIN:
#  endif
#  ifdef EWOULDBLOCK
		case EWOULDBLOCK:
#  endif
#endif

			return true;
	}

	return false;
}
