/*
 * Copyright (C) 2017 Aaron M. D. Jones <aaronmdjones@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INC_LIBACOPM_UTILS_LOG_H
#define INC_LIBACOPM_UTILS_LOG_H

#include "acopm-common.h"

#define ACOPM_LOGLVL_NONE       0x00
#define ACOPM_LOGLVL_CRITICAL   0x01
#define ACOPM_LOGLVL_ERROR      0x02
#define ACOPM_LOGLVL_WARNING    0x04
#define ACOPM_LOGLVL_NOTICE     0x08
#define ACOPM_LOGLVL_INFO       0x10
#define ACOPM_LOGLVL_TRAFFIC    0x20
#define ACOPM_LOGLVL_DEBUG      0x40

#define ACOPM_LOGLVL_RANGE_MIN  0x00
#define ACOPM_LOGLVL_RANGE_MAX  0x7F

#ifdef BUILD_WITH_DEBUG
#  define ACOPM_LOGLVL_DEFAULT  0x7F
#else
#  define ACOPM_LOGLVL_DEFAULT  0x1F
#endif

bool acopm_log_init(void);
bool acopm_log_set_file(const char *);
void acopm_log_set_mask(unsigned int);
bool acopm_log_file_is_set(void);
bool acopm_log_mask_is_set(void);
void acopm_log_flush(void);
void acopm_log_free(void);

void acopm_log_critical(const char *, ...) acopm_attr_printf(1, 2);
void acopm_log_error(const char *, ...) acopm_attr_printf(1, 2);
void acopm_log_warning(const char *, ...) acopm_attr_printf(1, 2);
void acopm_log_notice(const char *, ...) acopm_attr_printf(1, 2);
void acopm_log_info(const char *, ...) acopm_attr_printf(1, 2);
void acopm_log_traffic(const char *, ...) acopm_attr_printf(1, 2);

#ifndef NDEBUG
void acopm_log_debug(const char *, ...) acopm_attr_printf(1, 2);
#else
static inline void acopm_attr_inline acopm_attr_printf(1, 2)
acopm_log_debug(const char acopm_attr_unused *const restrict format, ...)
{
	/* Do nothing */
}
#endif

#endif /* !INC_LIBACOPM_UTILS_LOG_H */
