/*
 * Copyright (C) 2017 Aaron M. D. Jones <aaronmdjones@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.
 *
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INC_LIBACOPM_MISC_SIPHASH24_H
#define INC_LIBACOPM_MISC_SIPHASH24_H

#ifdef UTHASH_H
#  error "use siphash24.h *instead of* uthash.h"
#endif

#include "acopm-common.h"

#include "utils/log.h" /* acopm_log_critical() */

#define SIPHASH24_KEY_LENGTH 16 /* bytes */

bool acopm_siphash24_setkey(const uint8_t key[static SIPHASH24_KEY_LENGTH]);
uint64_t acopm_siphash24_digest(const void *, size_t) acopm_attr_pure;

#define HASH_FUNCTION(k, l, v)                                  \
    do {                                                        \
        (v) = (unsigned int) acopm_siphash24_digest((k), (l));  \
    } while(0)

#define uthash_fatal(msg) do { (void) acopm_log_critical("uthash: %s", (msg)); abort(); } while(0)

#include "3rdparty/uthash.h"

#endif /* INC_LIBACOPM_MISC_SIPHASH24_H */
